/* Copyright (c) 2011, Make, Hack, Void Inc
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of the Make, Hack, Void nor the
 *    names of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MAKE, HACK, VOID BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef MHV_EEPROM_H_
#define MHV_EEPROM_H_

#include <MHV_io.h>
#include <MHV_Lock.h>
#include <inttypes.h>

#define MHV_EEPROM_ASSIGN_INTERRUPTS(_mhvEeprom) \
ISR(MHV_EEPROM_VECT) { \
	_mhvEeprom.writeInterrupt(); \
}

#define MHV_EEPROM_CREATE(_mhvObjectName) \
		MHV_EEPROM _mhvObjectName; \
		MHV_EEPROM_ASSIGN_INTERRUPTS(_mhvObjectName);

#define MHV_EEPROM_BUSY	-1

class MHV_EEPROM {
private:
	MHV_Lock	_lock;
	uint16_t	_writeAddress;
	uint8_t		*_writeBuffer;
	uint16_t	_bytesWritten;
	uint16_t	_bytesToWrite;
	void		(*_doneCallback)(void *buffer, void *data);
	void		*_doneCallbackData;

public:
	MHV_EEPROM();
	int16_t read(uint16_t address);
	uint8_t busyRead(uint16_t address);
	int8_t read(void *buffer, uint16_t address, uint16_t length);
	void busyRead(void *buffer, uint16_t address, uint16_t length);
	int8_t write(uint16_t address, uint8_t data);
	int8_t busyWrite(void *buffer, uint16_t address, uint16_t length);
	int8_t write(void *buffer, uint16_t address, uint16_t length,
			void (*doneCallback)(void *buffer, void *data),
			void *doneCallbackData);
	void writeInterrupt();
	bool isBusy();
};

#endif /* MHV_EEPROM_H_ */
